package com.incubator.game.handler;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.data.po.PlayerInfoPO;
import com.incubator.game.player.Player;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * 登录游戏服
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.LoginReq_VALUE)
public class LoginGameHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.LoginRes_VALUE);

		// 判断参数
		String token = request.getDataValue("token", String.class, "");
//		if (token==null) {
//			//如果token为空则重新登录
//			Log.info("登录token为空");
//			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
//			return;
//		}

		String username = request.getDataValue("username", String.class, "");
		String userId = request.getDataValue("userId", String.class, "");

		// 创建player
		String player_id = UUID.randomUUID().toString();
		PlayerInfoPO playerInfoPO = new PlayerInfoPO();
		playerInfoPO.playerId = player_id;
		playerInfoPO.name = username;
		playerInfoPO.avatarUrl = "";
		playerInfoPO.sex = 1;
		playerInfoPO.exp = 3000;
		playerInfoPO.vipLevel = 1;
		playerInfoPO.masterScore = 0;
		playerInfoPO.masterScoreLevel = 0;
		playerInfoPO.gold = 1000;
		playerInfoPO.diamond = 2000;
		playerInfoPO.startTimeVip = "";
		playerInfoPO.endTimeVip = "";
		playerInfoPO.gps = "";
		playerInfoPO.gpsAddr = "";
		playerInfoPO.foca = 0;
		playerInfoPO.ranking = 2;
		playerInfoPO.figureId = 1;
		playerInfoPO.realName = "李晓强";
		playerInfoPO.phoneNum = "13800000001";
		playerInfoPO.idCard = "42010000000000000X";
		playerInfoPO.friendMasterNum = 0;
		playerInfoPO.friendMasterRate = 0;
		playerInfoPO.tradPoints = 0;
		playerInfoPO.tradPointsRate = 0;
		playerInfoPO.tradMatchNum = 0;
		playerInfoPO.tradMatchRate = 0;
		playerInfoPO.noReshNum = 0;
		playerInfoPO.noReshRate = 0;
//		playerInfoPO.token = token;
		Player player = PlayerUtil.createPlayer(playerInfoPO);

		// 绑定session
		player.bind(session, userId, player.getId(), token);
		Log.info("玩家进入游戏 playerId={}", player.getId());

		// 登录数据
		player.onLogin();

		// 正常返回
		Map<String, Object> map = new HashMap<>();
		map.put("playerInfo", ProtoUtil.playerInfoToMessage(player));
		// 玩家绑定的房间号
		map.put("roomId", player.data.roomId);
		// 玩家绑定的官方比赛id
		map.put("contestId", player.data.contestId);
		// 官方比赛信息列表
		map.put("contestInfo", ProtoUtil.contestListToMessage());


		response.setData(map);
	}
}
