package com.incubator.game.handler.contest;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.player.Player;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;

/**
 * 获取官方比赛详细信息 请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.MatchRoomListReq_VALUE)
public class GetContestRoomListHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.MatchRoomListRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断参数
        long contestId = request.getDataValue("contestId", Integer.class, 0);
        if (contestId <= 0L) {
            Log.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        // 比赛是否存在
        Contest contest = ContestService.getInstance().contestMap.getOrDefault(contestId, null);
        if (contest == null) {
            Log.info("比赛不存在或已结束");
            response.setCode(CommonProto.Code.CONTEST_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，比赛不存在或已结束...");
            return;
        }

        // 正常返回 比赛报名列表
        Map<String, Object> data = new HashMap<>();

//        for (Map.Entry<Integer, Map<Long, List<Struct. PromotionData>>> entry : contest.data.promotionMap.entrySet()) {
//            int round = entry.getKey();
//            Map<Long, List<Struct. PromotionData>> roomMap = entry.getValue();
//            for (Map.Entry<Long, List<Struct. PromotionData>> roomMapEntry : entry.getValue().entrySet()) {
//                roomMapEntry.getKey();
//                roomMapEntry.getValue();
//            }
//
//            data.put("round", round);
//        }

        response.setData(data);
    }
}
