package com.incubator.game.handler.jdgd;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.RoomService;
import com.incubator.game.room.JDGDRoom;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.Arrays;
import java.util.Map;
import java.util.Objects;

/**
 * 掼蛋 过请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.PlayNoCardsReq_VALUE)
public class JDGDPassHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.PlayNoCardsRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            Log.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断玩家绑定房间
        if (player.data.roomId <= 0) {
            Log.info("准备失败，玩家不在房间");
            response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家不在房间...");
            return;
        }

        // 玩家房间是否存在
        JDGDRoom room = (JDGDRoom) RoomService.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            Log.info("玩家房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家房间已解散...");
            return;
        }

        // 当前状态是否可操作
        if (room.state != 2 || player.data.state != 2) {
            Log.info("房间或玩家不在游戏中状态...");
            response.setCode(7);
            response.setMessage("操作失败，房间或玩家不在游戏中状态...");
            return;
        }

        // 当前可操作玩家是否自己
        if (!room.data.currentPlayer.getId().equals(player.getId())) {
            Log.info("当前不是自己出牌回合...");
            response.setCode(8);
            response.setMessage("操作失败，当前不是自己出牌回合...");
            return;
        }

        // 该自己出牌 不可不出
        if (room.data.disCardPlayer == null && player.getId().equals(room.data.currentPlayer.getId())) {
            Log.info("当前是自己出牌回合...");
            response.setCode(9);
            response.setMessage("操作失败，当前是自己出牌回合...");
            return;
        }

        // 接风玩家 不可不出
        if (room.data.maxNum == 4) {
            if (room.data.disCardPlayer != null && !player.getId().equals(room.data.disCardPlayer.getId())
                    && room.data.jieFengPlayer != null && room.data.passList.isEmpty()
                    && player.getId().equals(room.data.jieFengPlayer.getId())) {
                Log.info("当前是自己出牌回合...");
                response.setCode(9);
                response.setMessage("操作失败，当前是自己出牌回合...");
                return;
            }
        }

        // 3家要不起 不可不出
        if (room.data.disCardPlayer != null && player.getId().equals(room.data.disCardPlayer.getId())) {
            Log.info("当前是自己出牌回合...");
            response.setCode(9);
            response.setMessage("操作失败，当前是自己出牌回合...");
            return;
        }

        Log.info("过...玩家ID:{}, 位置:{}, 手牌大小:{}, 手牌信息: {}", player.getId(), player.data.pos, player.data.remainCards.length, Arrays.toString(player.data.remainCards));

        boolean isClear = false;

        // 添加过玩家列表
        if (!room.data.passList.contains(player.getId())) {
            room.data.passList.add(player.getId());
        }

        // 4人房3人不出，清空出牌堆
        if (!room.data.passList.isEmpty() && room.data.passList.size() >= room.data.curOpPlayerNum - 1) {
            room.data.curDisCardList = new int[0];
            room.data.curDisCardType = null;
            room.data.disCardPlayer = null;
            room.data.curOpPlayerNum = room.getCanOpPlayer();
            // 清空过玩家集合
            room.data.passList.clear();
            isClear = true;
        }

        // todo 判断接风
        if (isClear && room.data.jieFengPlayer != null) {
            room.data.currentPlayer = room.actors.getOrDefault(room.data.jieFengPlayer.getId(), null);
        } else {
            // 设置下一个操作玩家
            room.data.currentPlayer = room.getNextPlayer(player);
        }

        // 广播其他3人
        for (Player tmPlayer : room.data.playerMap.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = ProtoUtil.roomToMessage(room, tmPlayer, room.data.currentPlayer);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }

        // 正常返回
        Map<String, Object> data = ProtoUtil.roomToMessage(room, player, room.data.currentPlayer);
        response.setData(data);
    }
}
