package com.incubator.game.handler.jdgd;

import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.JDGDRoom;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.JDGDUtils;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.util.Arrays;
import java.util.Map;
import java.util.Objects;

/**
 * 掼蛋 进还贡请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.EnterBackCardsReq_VALUE)
public class JDGDTributeHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.EnterBackCardsRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("操作失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败...");
			return;
		}

		// 判断玩家绑定房间
		if (player.data.roomId <= 0) {
			Log.info("准备失败，玩家不在房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家不在房间...");
			return;
		}

		// 玩家房间是否存在
		JDGDRoom room = (JDGDRoom) RoomService.getInstance().getRoom(player.data.roomId);
		if (room == null) {
			Log.info("玩家房间已解散");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家房间已解散...");
			return;
		}

		// 房间是否已开始
		if (room.state != 0) {
			Log.info("玩家房间已开始，不可准备");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("玩家房间已开始，不可准备...");
		}

		// 判断参数
		int card = request.getDataValue("card", int.class, 0);
		if (!Arrays.asList(JDGDUtils.values).contains(card)) {
			Log.info("参数错误");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误...");
			return;
		}

		// 判断手牌
		if (!CardUtils.contains(player.data.remainCards, card)) {
			Log.info("参数错误，玩家出的牌不在手上...");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误，玩家出的牌不在手上...");
			return;
		}

		// 进贡者判断
		if (room.data.tributeData.tributeList != null && room.data.tributeData.tributeList.contains(player.getId())) {
			// 是否是玩家手牌中最大的牌

		}

		// 还贡者判断 是否是10以内的牌


		// 进还贡逻辑


		// 广播房间其他3个玩家
		for (Player tmPlayer : room.actors.values()) {
			if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
				Map<String, Object> data = ProtoUtil.roomToMessage(room, tmPlayer, null);
				tmPlayer.receive(CommonProto.Cmd.ReadyStateUpdate_VALUE, data);
			}
		}

		// 正常返回
		response.setData(ProtoUtil.roomToMessage(room, player, null));
	}
}
