package com.incubator.game.handler.room;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.MessageHandler;
import com.incubator.common.log4j.Log;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 创建房间请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.CreateRoomReq_VALUE)
public class CreatRoomHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.CreateRoomRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			Log.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 判断参数
		int[] type = request.getDataValue("type", int[].class, new int[]{});
		if (type.length != 11) {
			Log.info("参数错误");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误");
			return;
		}


		// 玩家是否存在房间
		Room room = RoomService.getInstance().getRoomByPlayerId(player.getId());
		if (room != null) {
			Log.info("玩家已在房间，不可创建房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_EXIST_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 房间规则
		JSONObject jsonData = new JSONObject();
		// 1.掼蛋
		jsonData.put("gameType", type[0]);
		// 玩法1 1:连续打，2:按局数，3:按时间
		jsonData.put("playingTypeOne", type[1]);
		// 玩法2 1:A必打 2:J必打 3:8必打 4:5必打
		jsonData.put("playingTypeTwo", type[2]);
		// 升级规则 1:123级，2:124级，3:135级，4:246级
		jsonData.put("upgradeType", type[3]);
		// 翻山规则 1:有头游无末游过A结束，2:有头游过A结束，3:3局不过A判负结束，4:3局不过A从2开始
		jsonData.put("flippingType", type[4]);
		// 进贡规则 1:进贡，2:不进贡（头游先出）
		jsonData.put("advanceType", type[5]);
		// 超时规则 1:过牌 2:待机 3:托管
		jsonData.put("timeoutType", type[6]);
		// 语音规则 1允许 2禁用
		jsonData.put("voiceType", type[7]);
		// 记牌器 1允许 2禁用
		jsonData.put("recordType", type[8]);
		// 观看规则 1第一视角 2上帝视角 3上帝&第一视角 4禁用
		jsonData.put("watchType", type[9]);
		// 人数模式 1:2人 2:4人
		int playerNum = type[10] == 1? 2 : (type[10] == 2? 4 : 0);
		jsonData.put("PlayerNum", playerNum);
		// 房间最大局数
		jsonData.put("maxRound", 8);
		// 房间类型 0:好友房 1:官方比赛房
		jsonData.put("type", 0);
		// 房间id
		jsonData.put("roomId", RoomService.getInstance().randomRoomId());

		// 创建房间
		room = RoomService.getInstance().creatRoom(player, 1, jsonData);

		if (room == null) {
			Log.info("系统错误");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 玩家绑定房间
		player.data.roomId = room.roomId;

		// 正常返回
		response.setData(ProtoUtil.roomToMessage(room, player, null));
	}
}
