package com.incubator.game.room;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.game.player.Player;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 房间实体基类
 */
public class Room {

    /** 房间规则数据 **/
    public JSONObject jsonData;

    /** 房间号 */
    public long roomId;
    /** 房间类型 0:好友房 1:官方比赛房 **/
    public int type;
    /** 房间玩法类型 1:掼蛋 */
    public int gameType;
    /** 房主playerId **/
    public String ownerId = "";
    /** 房间角色 [key:玩家id, value:玩家对象]**/
    public Map<String, Player> actors = new ConcurrentHashMap<>();
    /** 牌库 **/
    public List<Integer> cardList = new ArrayList<>();

    /** 激活标记 **/
    private boolean isActive;
    /** 检测标记 **/
    public boolean flag;
    /** 时间秒数 **/
    public int time;
    /** 房间状态(0等待 1开局 2进行中 3已结束) **/
    public int state;

    /** 房间任务主线程 **/
    private ScheduledExecutorService scheduler;

    public Room() {
        this.scheduler = Executors.newSingleThreadScheduledExecutor();
    }

    /** 初始化 **/
    public void init(Player player, JSONObject jsonData) {}

    /** 加入房间 **/
    public void joinRoom(Player player) {}

    /**
     * 开启房间主线程任务
     */
    public void startRoomTask() {
        this.isActive = true;
        this.scheduler.scheduleAtFixedRate(() -> {
            if (this.isActive) {
                this.doUpdate();
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    /**
     * 房间主线程逻辑
     */
    void doUpdate() {}

    /**
     * 房间重置逻辑
     */
    void resetRoom() {}

    /**
     * 重置房间，用于对象池回收前清理状态
     */
    public void reset() {
        this.isActive = false;
    }

    /**
     * 销毁房间，释放资源
     */
    public void destroy() {
        this.reset();
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 从缓存移除
        RoomService.getInstance().roomMap.remove(this.roomId);
        // 重置房间数据
        this.resetRoom();
        // 归还房间对象
        RoomService.getInstance().roomPool.releaseRoom(this);
        Log.debug("Room {} has been destroyed.", this.roomId);
    }
}
