package com.incubator.game.room;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.player.Player;

import java.util.*;
import java.util.concurrent.*;

/**
 * 房间 管理类
 */
public class RoomService {

    private static RoomService instance;
    /** 房间对象池 **/
    public RoomPool roomPool;
    /** 房间缓存 [key:房间id, value:房间实例] **/
    public Map<Long, Room> roomMap = new ConcurrentHashMap<>();

    public synchronized static RoomService getInstance() {
        if (instance == null) {
            instance = new RoomService();
        }
        return instance;
    }

    /**
     * 初始化
     *
     * @param maxIdleRooms 最大空闲房间数量
     */
    public void init(int maxIdleRooms) {
        Log.info("初始化房间服务...");
        this.roomPool = new RoomPool(maxIdleRooms);
    }

    /**
     * 每日刷新
     */
    public void refreshNewDay() {

    }

    /**
     * 生成唯一6位数房间号
     *
     * @return
     */
    public long randomRoomId() {
        long roomNumber;
        // 循环生成，直到找到一个唯一的房间号
        do {
            roomNumber = RandomUtil.randomSixDigits();
        } while (this.roomMap.containsKey(roomNumber));
        return roomNumber;
    }

    /**
     * 生成唯一7位数赛事房间号
     *
     * @return
     */
    public long randomRoomIdByContest() {
        long roomNumber;
        // 循环生成，直到找到一个唯一的房间号
        do {
            roomNumber = RandomUtil.randomSevenDigits();
        } while (this.roomMap.containsKey(roomNumber));
        return roomNumber;
    }

    /**
     * 获取房间
     *
     * @param roomId
     * @return
     */
    public Room getRoom(long roomId) {
        return this.roomMap.getOrDefault(roomId, null);
    }

    /**
     * 获取玩家房间
     *
     * @param playerId
     * @return
     */
    public Room getRoomByPlayerId(String playerId) {
        for (Room room : this.roomMap.values()) {
            if (room != null && room.actors != null && room.actors.containsKey(playerId)) {
                return room;
            }
        }
        return null;
    }

    /**
     * 创建房间
     *
     * @param player    创建房间玩家
     * @param type      房间类型 0:好友房 1:官方比赛房
     * @param jsonData  房间规则数据
     * @return
     */
    public synchronized Room creatRoom(Player player, int type, JSONObject jsonData) {
        // 从对象池获取房间
        Room room = this.roomPool.acquireRoom(type);
        // 初始化房间
        room.init(player, jsonData);
        // 房主加入房间
        room.joinRoom(player);
        // 加入管理映射
        this.roomMap.put(room.roomId, room);
        return room;
    }
}
